# flake8: noqa

from ...cs_models.resources.Claim.models import ClaimModel
from ...cs_models.resources.ClaimChallenged.models import (
    ClaimChallengedModel,
)
from ...cs_models.resources.ClaimDependency.models import (
    ClaimDependencyModel,
)
from ...cs_models.resources.Company.models import CompanyModel
from ...cs_models.resources.CrossRef.models import CrossRefModel
from ...cs_models.resources.DocumentReference.models import (
    DocumentReferenceModel,
)
from ...cs_models.resources.File.models import FileModel
from ...cs_models.resources.InstitutionProbability.models import (
    InstitutionProbabilityModel,
)
from ...cs_models.resources.OrangeBookPatent.models import (
    OrangeBookPatentModel,
)
from ...cs_models.resources.OrangeBookProduct.models import (
    OrangeBookProductModel,
)
from ...cs_models.resources.PacerCase.models import (
    PacerCaseModel,
)
from ...cs_models.resources.Patent.models import (
    PatentModel,
)
from ...cs_models.resources.PatentApplication.models import (
    PatentApplicationModel,
)
from ...cs_models.resources.PatentCompound.models import (
    PatentCompoundModel,
)
from ...cs_models.resources.PatentIdentifierSynonym.models import (
    PatentIdentifierSynonymModel,
)
from ...cs_models.resources.PriorArt.models import (
    PriorArtModel,
)
from ...cs_models.resources.PriorArtRefRelation.models import (
    PriorArtRefRelationModel,
)
from ...cs_models.resources.PTAB2Decision.models import (
    PTAB2DecisionModel,
)
from ...cs_models.resources.PTAB2Document.models import (
    PTAB2DocumentModel,
)
from ...cs_models.resources.PTAB2Proceeding.models import (
    PTAB2ProceedingModel,
)
from ...cs_models.resources.RelatedMatter.models import (
    RelatedMatterModel,
)
from ...cs_models.resources.TrialStat.models import (
    TrialStatModel,
)
from ...cs_models.resources.UserSearchHistoryItem.models import (
    UserSearchHistoryItemModel,
)
